package vc.thinker.mqtt.proxyclient.enums;

import lombok.AllArgsConstructor;
import lombok.Getter;
import vc.thinker.absctacts.mqtt.topic.TopicDefinition;
import vc.thinker.mqtt.proxyclient.actor.down.MeterChangeActor;
import vc.thinker.mqtt.proxyclient.actor.down.MeterListPullResponseActor;
import vc.thinker.mqtt.proxyclient.actor.down.ThirdPartyServiceExecuteActor;
import vc.thinker.mqtt.proxyclient.actor.up.MeterListPullActor;
import vc.thinker.mqtt.proxyclient.actor.up.ThirdPartyReportActor;
import vc.thinker.mqtt.proxyclient.actor.up.ThirdPartyServiceExecuteResponseActor;
import vc.thinker.utils.Resolve;

/**
 * Topic-actor 定义
 *
 * @author HeTongHao
 * @since 2022/8/10 11:16
 */
@Getter
@AllArgsConstructor
public enum ThirdPartyProxyTopicDefinition implements TopicDefinition {
    // 数据上报
    report("third-party/+/v2/up/report", ThirdPartyReportActor.class),
    // 服务执行
    executeService("third-party/+/down/execute-service", ThirdPartyServiceExecuteActor.class),
    executeServiceResponse("third-party/+/up/execute-service/response", ThirdPartyServiceExecuteResponseActor.class),
    // 仪表同步
    meterListPull("third-party/+/up/meter-list-pull", MeterListPullActor.class),
    meterListPullResponse("third-party/+/down/meter-list-pull/response", MeterListPullResponseActor.class),
    meterChange("third-party/+/down/meter-change", MeterChangeActor.class),
    ;
    private final String topic;
    private final Class<?> actorClass;

    public String genTopic(String protocolCode) {
        return this.topic.replace("+", protocolCode);
    }


    /**
     * 根据topic解析第三方协议编号
     *
     * @param topic topic
     * @return 第三方协议编号
     */
    public static String resolveProtocolCodeByTopic(String topic) {
        return topic.split("/")[1];
    }

    public static final Resolve<ThirdPartyProxyTopicDefinition, String> RESOLVE_BY_TOPIC =
            new Resolve<>(values(), ThirdPartyProxyTopicDefinition::getTopic);
}
